/* PYVMS__TEST_ROUTINES.C -- 04-SEP-1999 Uwe Zessin
   tests for internal quad/octaword to/from Python longword conversion routines

29-JUL-1998 ZE. -- test-interface for quad2PYlong-integer conversion tests
04-AUG-1998 ZE. -- test-interface for PYlong-integer2quad conversion tests
27-DEC-1998 ZE. -- test-interface for Uquad2PYlong-integer conversion tests
09-JAN-1999 ZE. -- test-interface for octa2PyLong conversion tests
09-JAN-1999 ZE. -- test-interface for PyLong2octa conversion tests
11-MAY-1999 ZE. -- FB- pyvms_test__pyl2o, o_octaword[4] (was [2])
04-SEP-1999 ZE. -- move test routines from PYVMSMODULE.C to here
23-JUL-2000 ZE. -- add some test cases in documentation

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  PYVMS__TEST_ROUTINES

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module PYVMS__TEST_ROUTINES "20000723"
#endif

#include "vmsdef.h"		/* includes "Python.h" */

/* ------------------------------------------------------------------------- */
/* "py-long = test_o2pyl (q3, q2, q1, q0)"*/

extern PyObject * vms__cvt_octa2pylong (long q_octaword[4]);

PyObject * pyvms_test__o2pyl (PyObject *self, PyObject *args)
{
	long	o_l3;
	long	o_l2;
	long	o_l1;
	long	o_l0;
	long	q_octaword[4];

	long	l_error;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "llll", &o_l3, &o_l2, &o_l1, &o_l0))
	{
	    return NULL;
	}

	/* -------------------- */
	q_octaword[0] = o_l0;
	q_octaword[1] = o_l1;
	q_octaword[2] = o_l2;
	q_octaword[3] = o_l3;

	/* -------------------- */
	return vms__cvt_octa2pylong (&q_octaword[0]);
} /* pyvms_test__o2pyl () */

/*
$python
import pyvms
def to (q3,q2,q1,q0):
  print pyvms.test__o2pyl(q3,q2,q1,q0), hex(pyvms.test__o2pyl(q3,q2,q1,q0))
#

#
to (0,0,0,0)
to (0,0,0,1)
to (0,0,0,0xf)
to (0,0,0,0x7f)
to (0,0,0,0xff)
to (0,0,0,0x17f)
to (0,0,0,0x80)
to (0,0,0,0x4080)
to (0,0,0,0x78787878)
to (0,0,0,0x80808080)
to (0,0,0,0x75555555)
to (0,0,0,0x76545555)
to (0,0,0,0xe5555555)
to (0,0,0x1,0x75555555)
to (0,0,0x2,0x75555555)
to (0,0,0x55555,0x75555555)

to (0x12345678,0x9abcdefe,0x12345678,0x9abcdefe)
to (0xffffffff,0xffffffff,0xffffffff,0xfffffffe)
to (0xffffffff,0xffffffff,0xffffffff,0x80000000)
to (0xffffffff,0xffffffff,0xffffffff,0x00000000)
to (0xffffffff,0xffffffff,0xfffffffe,0x00000000)
to (0xffffffff,0xffffffff,0x80000000,0x00000000)
to (0xffffffff,0xffffffff,0x80000000,0x00000001)
to (0xffffffff,0xffffffff,0x80000000,0x00000000)
to (0xffffffff,0xffffffff,0x00000000,0x00000000)
to (0xffffffff,0xfffffffe,0x00000000,0x00000000)
to (0xffffffff,0x80000000,0x00000000,0x00000000)
to (0xffffffff,0x00000000,0x00000000,0x00000000)
to (0xfffffffe,0x00000000,0x00000000,0x00000000)
to (0xf0000000,0x00000000,0x00000000,0x00000000)
to (0x80000000,0x00000000,0x00000000,0x00000000)
to (0x80000000,0x00000000,0x00000000,0x00000001)
*/

/* ------------------------------------------------------------------------- */

/* "(q3, q2, q1, q0) = test_pyl2o (py-long)" */

extern long vms__cvt_pylong2octa (PyObject * r_longint, long * o_octaword);

PyObject * pyvms_test__pyl2o (PyObject *self, PyObject *args)
{
	long		  o_octaword[4];
	PyObject	* ar_pylong;

	long		  l_error;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "O", &ar_pylong))
	{
	    return NULL;
	}

	/* -------------------- */
	if (!PyLong_Check(ar_pylong))
	{
	    PyErr_SetString(PyExc_TypeError,
		"argument 1: must be long integer");
	    return NULL;
	}

	/* -------------------- */
	o_octaword[0] = 0;
	o_octaword[1] = 0;
	o_octaword[2] = 0;
	o_octaword[3] = 0;

	l_error = vms__cvt_pylong2octa (ar_pylong, &o_octaword[0]);
	/* if l_error ... -
	    PyErr_SetString(conversion failed)
	    return NULL
	*/

	/* -------------------- */
	return Py_BuildValue ("(llll)",
	    o_octaword[3], o_octaword[2], o_octaword[1], o_octaword[0]);
} /* pyvms_test__pyl2o () */

/*
$python
import pyvms
def p(l1,l2,l3,l4):
  print (hex(l1),hex(l2),hex(l3),hex(l4))
# -p

a,b,c,d = pyvms.test__pyl2o (0L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (1L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x7fffffffL) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x80000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x87654321L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0xffffffffL) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x100000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x180000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x187654321L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x1ffffffffL) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x7fffffffffffffffL) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x8000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x8765432187654321L) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (0x18000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x18765432187654321L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x800000000000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x876543218765432187654321L) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (0x1800000000000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x1876543218765432187654321L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x7fffffff800000000000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x76543218876543218765432187654321L) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (0x80000000000000000000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0x80000000000000000000000000000001L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0xffffffffffffffffffffffffffffffffL) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (0xfffffffffffffffffffffffffffffffeL) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (-1L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-2L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0xfL) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (-0x7fffffffL) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x80000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x100000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x200000000L) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (-0x7fffffff00000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x7fffffffffffffffL) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (-0x8000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x8000000000000001L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x8000000000000002L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x80000000ffffffffL) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x8000000100000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x8fffffffffffffffL) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (-0x10000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x10000000000000001L) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (-0x7fffffffffffffffffffffffL) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x800000000000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0xffffffffffffffffffffffffL) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (-0x1000000000000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x1000000000000000000000001L) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (-0x76543218000000000000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x76543218876543218765432187654321L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x7fffffffffffffffffffffffffffffffL) ; p(a,b,c,d)

a,b,c,d = pyvms.test__pyl2o (-0x12345678123456781234567812345678L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x80000000000000000000000000000000L) ; p(a,b,c,d)
a,b,c,d = pyvms.test__pyl2o (-0x80000000000000000000000000000001L) ; p(a,b,c,d)
*/

/* ------------------------------------------------------------------------- */

/* "(q-hi, q-lo) = test_pyl2q (py-long)" */

extern long vms__cvt_pylong2quad (PyObject * r_longint, long * q_quadword);

PyObject * pyvms_test__pyl2q (PyObject *self, PyObject *args)
{
	long		  q_quadword[2];
	PyObject	* ar_pylong;

	long		  l_error;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "O", &ar_pylong))
	{
	    return NULL;
	}

	/* -------------------- */
	if (!PyLong_Check(ar_pylong))
	{
	    PyErr_SetString(PyExc_TypeError,
		"argument 1: must be long integer");
	    return NULL;
	}

	/* -------------------- */
	q_quadword[1] = 0;
	q_quadword[0] = 0;

	l_error = vms__cvt_pylong2quad (ar_pylong, &q_quadword[0]);
	/* if l_error ... -
	    PyErr_SetString(conversion failed)
	    return NULL
	*/

	/* -------------------- */
	return Py_BuildValue ("(ii)", q_quadword[1], q_quadword[0]);
} /* pyvms_test__pyl2q () */

/*
>>> import pyvms
def p(l1,l2):
  print ( (l1,l2) , (hex(l1),hex(l2)) )
# -p

a,b = pyvms.test__pyl2q (0L) ; p(a,b)
a,b = pyvms.test__pyl2q (1L) ; p(a,b)
a,b = pyvms.test__pyl2q (0x7fffffffL) ; p(a,b)
a,b = pyvms.test__pyl2q (0x80000000L) ; p(a,b)
a,b = pyvms.test__pyl2q (0xffffffffL) ; p(a,b)
a,b = pyvms.test__pyl2q (0x100000000L) ; p(a,b)
a,b = pyvms.test__pyl2q (0x180000000L) ; p(a,b)
a,b = pyvms.test__pyl2q (0x1ffffffffL) ; p(a,b)
a,b = pyvms.test__pyl2q (0x7fffffffffffffffL) ; p(a,b)
a,b = pyvms.test__pyl2q (0x8000000000000000L) ; p(a,b)
a,b = pyvms.test__pyl2q (-1L) ; p(a,b)
a,b = pyvms.test__pyl2q (-0xfL) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x7fffffffL) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x80000000L) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x100000000L) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x200000000L) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x7fffffff00000000L) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x7fffffffffffffffL) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x8000000000000000L) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x8000000000000001L) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x8000000000000002L) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x80000000ffffffffL) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x8000000100000000L) ; p(a,b)
a,b = pyvms.test__pyl2q (-0x8fffffffffffffffL) ; p(a,b)
>>>
*/

/* ------------------------------------------------------------------------- */

/* "py-long = test_q2pyl (q-hi, q-lo)"*/

extern PyObject * vms__cvt_quad2pylong (long q_quadword[2]);

PyObject * pyvms_test__q2pyl (PyObject *self, PyObject *args)
{
	long	q_hi;
	long	q_lo;
	long	q_quadword[2];

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "ll", &q_hi, &q_lo))
	{
	    return NULL;
	}

	/* -------------------- */
	q_quadword[1] = q_hi;
	q_quadword[0] = q_lo;

	/* -------------------- */
	return vms__cvt_quad2pylong (q_quadword);
} /* pyvms_test__q2pyl () */
/*
$python
import pyvms
def tq (h,l):
  print pyvms.test__q2pyl(h,l), hex(pyvms.test__q2pyl(h,l))
#

#
tq (0,0)
tq (0,1)
tq (0,0xf)
tq (0,0x7f)
tq (0,0xff)
tq (0,0x17f)
tq (0,0x80)
tq (0,0x4080)
tq (0,0x78787878)
tq (0,0x80808080)
tq (0,0x75555555)
tq (0,0x76545555)
tq (0,0xe5555555)
tq (0x1,0x75555555)
tq (0x2,0x75555555)
tq (0x55555,0x75555555)
tq (1,0x80808080)
tq (0xffffffff,0xffffffff)
tq (0xffffffff,0xfffffffe)
tq (0xffffffff,0x00000000)
tq (0x90000000,0x00000000)
tq (0x80000000,0x00000001)
tq (0x80000000,0x00000000)
*/

/* ------------------------------------------------------------------------- */

/* "py-long = test_uq2pyl (q-hi, q-lo)"*/

extern PyObject * vms__cvt_uquad2pylong (unsigned long q_quadword[2]);

PyObject * pyvms_test__uq2pyl (PyObject *self, PyObject *args)
{
	unsigned long	q_hi;
	unsigned long	q_lo;
	unsigned long	q_quadword[2];

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "ll", &q_hi, &q_lo))
	{
	    return NULL;
	}

	/* -------------------- */
	q_quadword[1] = q_hi;
	q_quadword[0] = q_lo;

	/* -------------------- */
	return vms__cvt_uquad2pylong (q_quadword);
} /* pyvms_test__uq2pyl () */

/*
>>> import pyvms
hex( pyvms.test__uq2pyl (0,0)           )
hex( pyvms.test__uq2pyl (0,1)           )
hex( pyvms.test__uq2pyl (0,0x7fffffff)  )
hex( pyvms.test__uq2pyl (0,0x80000000)  )
hex( pyvms.test__uq2pyl (0,0xffffffff)  )
hex( pyvms.test__uq2pyl (0,0x100000000) ) # err
hex( pyvms.test__uq2pyl (1,0)           )
hex( pyvms.test__uq2pyl (0x7fffffff,0)  )
hex( pyvms.test__uq2pyl (0x80000000,0)  )
hex( pyvms.test__uq2pyl (0x80000000,1)  )
hex( pyvms.test__uq2pyl (0xffffffff,0)  )
hex( pyvms.test__uq2pyl (0xffffffff,0x80000000) )
hex( pyvms.test__uq2pyl (0x7fffffff,0xffffffff) )
hex( pyvms.test__uq2pyl (0xffffffff,0xffffffff) )
hex( pyvms.test__uq2pyl (0x100000000,0)  )
*/

/* ------------------------------------------------------------------------- */

/* EOF: PYVMS__TEST_ROUTINES.C */
