/* PYVMSMODULE.C -- VMS/Python -- 19-NOV-1997 Uwe Zessin

The 'pyvms' module provides access to vms-related components:
C-RTL functions, command history, item codes (fac$_name), bitmasks (fac$M_name),
constants (fac$C_name or fac$K_name) and other information.

   29-NOV-1997 ZE. -- changes for Python V1.5B1
   02-DEC-1997 ZE. -- merge VMSDEF into PYVMS
   14-DEC-1997 ZE. -- Include LIB$ROUTINES.H + SMG$ROUTINES.H.
   14-NOV-1998 ZE. -- interface to pyvms_uaf_get_usernames
			- code in PYVMS__UAF_ROUTINES.C
   31-DEC-1998 ZE. -- begin pyvms_crtl_open ()
   20-JAN-1999 ZE. -- test-interface for vmsobj_iosb_new()
   24-JAN-1999 ZE. -- test-interface for vmsobj_fab_new()
   25-JAN-1999 ZE. -- test-interface for vmsobj_rab_new()
   03-FEB-1999 ZE. -- test-interface for vmsobj__membuf_new()
   07-FEB-1999 ZE. -- test-interface for vmsobj_xaball_new()
   10-FEB-1999 ZE. -- test-interface for vmsobj_nam_new() + vmsobj_xabdat_new()
   13-FEB-1999 ZE. -- test-interface for vmsobj_xabfhc_new()
   18-FEB-1999 ZE. -- test-interface for vmsobj_xabitm_new()
   22-FEB-1999 ZE. -- test-interface for vmsobj_xabkey_new()
   06-MAR-1999 ZE. -- interface to pyvms_crtl_from_vms() + pyvms_crtl_to_vms()
			- code in PYVMS__CRTL_ROUTINES.C
			move pyvms_crtl_open() there, too
   08-MAR-1999 ZE. -- test-interface for vmsobj_xabpro_new() +
			vmsobj_xabrdt_new() + vmsobj_xabsum_new()
   13-MAR-1999 ZE. -- test-interface for vmsobj_xabtrm_new()
   15-MAR-1999 ZE. -- move vmsobj_* test-interface to PYVMS__VMSOBJ_ROUTINES.C
   29-MAY-1999 ZE. -- test-interface for vmsobj_lksb_new()
   07-JUL-1999 ZE. -- fix memory leak in pyvms_item_get()
   21-AUG-1999 ZE. -- test-interface for vmsobj__ownership_names_new()
   04-SEP-1999 ZE. -- pyvms_methods[] is now configured via data file
		      PYVMS__METHODS.DAT - see 'General Manual', configuration
*/

#ifdef __DECC
#pragma module PYVMSMODULE "20000221"
#endif

/* ------------------------------------------------------------------------- */

#include "python.h"
/* includes also: <string.h>, <errno.h> */
#include "vmsdef.h"

#include <lib$routines.h>	/* LIB$name	 */
#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_NAME */

/* ------------------------------------------------------------------------- */

/* numeric value of current VMS version (PYVMS_GLOBAL.C) */
extern vmsdef_xw_vmsver vmsdef_gw_vmsver;

/* VMS version string to numeric code translation table (VMSDAT_VMSVER.MAR) */
/* @unused, yet -- extern struct vmsdef_xr_vmsvertbl VMSDEF_GR_VMSDAT_VMSVER[];*/

/* list of $xxxDEF */
extern struct vmsdef_xr_defdirtbl VMSDEF_GR_DIRECTORY[];

/* ------------------------------------------------------------------------- */

#include "pyvms_readline.h"

/* SMG$ support */
extern long				pyvms_gl_keyboard_id;	/* SMG$ */
extern long				pyvms_gl_key_table_id;	/* SMG$ */
extern struct pyvms_xr_rlbuffer*	pyvms_gar_rlbuffer;	/* listhead */
/* ------------------------------------------------------------------------- */

PyObject *pyvms_gr_error;		/* exception pyvms.error */

static char pyvms__doc__ [] =
"The 'pyvms' module provides access to vms-related components:\n\
command history, item codes (fac$_name), bitmasks (fac$M_name) and\n\
other information.";

static char pyvms_t_cant_init [] =
"can't initialize module pyvms";

static char pyvms_t_cant_deferr [] =
"can't define pyvms.error";

static char pyvms_t_cant_defvms [] =
"can't define pyvms.vms_version_number";

/* ------------------------------------------------------------------------- */

/* delete 'l_delete_lines' from the beginning of the recall buffer */

/* Note: this does not change the contents of the SMG$ buffer */

long pyvms__history_delete (long l_delete_lines)
{
	struct pyvms_xr_rlbuffer	* ar_rlbuffer;
	unsigned long			  l_deleted_count;

	if (pyvms_gar_rlbuffer == NULL)
	{
	    return 0;		/* nothing in the buffer - nothing deleted */
	}

	l_deleted_count = 0;
	while (l_deleted_count < l_delete_lines)
	{
	    if (pyvms_gar_rlbuffer == NULL)
	    {
		break;				/* no more lines to delete */
	    }

	    ar_rlbuffer = pyvms_gar_rlbuffer;
	    /* remove first element from list */
	    pyvms_gar_rlbuffer = ar_rlbuffer->ar_next_rlbuff;
	    (void) free (ar_rlbuffer->at_line);		/* drop line */
	    if (ar_rlbuffer->at_prompt != NULL)
	    {
		(void) free (ar_rlbuffer->at_prompt);	/* drop prompt */
	    }
	    (void) free (ar_rlbuffer);		/* drop descriptor */
	    l_deleted_count++;			/* one more line deleted */
	}

	return l_deleted_count;			/* number of deleted lines */
} /* long pyvms__history_delete (long l_delete_lines) */

/* ------------------------------------------------------------------------- */

/* PYVMS methods */

/* d = pyvms.history_delete(n);
     n = 0 -> delete all
     n > 0 -> delete from begin of list (oldest lines first)
     n < 0 -> delete from end   of list (newest lines)
   d -> number of deleted lines
*/
PyObject * pyvms_history_delete (PyObject *self, PyObject *args)
{
	struct pyvms_xr_rlbuffer	* ar_rlbuffer;
	unsigned long			  l_deletecount;
	unsigned long			  l_deleted_count;
	unsigned long			  l_status;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i", &l_deletecount))
	{
	    return NULL;
	}

	/* -------------------- */
	if (pyvms_gar_rlbuffer == NULL)
	{
	    /* nothing in the buffer - so nothing deleted */
	    return PyInt_FromLong((long)0);
	}

	/* -------------------- */
	if (l_deletecount == 0)
	{
	    /* delete current virtual keyboard (and the recall buffer) */
	    l_status = smg$delete_virtual_keyboard(&pyvms_gl_keyboard_id);
	    /* @@ ignore any errors */

	    /* re-create virtual keyboard */
	    l_status = smg$create_virtual_keyboard (&pyvms_gl_keyboard_id);
	    /* this call MUST succeed ! */
	    if (l_status != SS$_NORMAL)
	    {
		printf ("%%PYVMS_history_delete(): SMG$CREATE_VIRTUAL_KEYBOARD() failed\n");
		lib$stop (l_status);
	    }

	    /* delete whole buffer - I doubt any one uses THAT MANY lines! */
	    l_deletecount = 0x7fffffff;
	    l_deleted_count = pyvms__history_delete (l_deletecount);
	    return PyInt_FromLong(l_deleted_count);
	}

	/* -------------------- */
	if (l_deletecount > 0)
	{
	    /* delete number of given lines from BEGIN of buffer */
	    l_deleted_count = pyvms__history_delete (l_deletecount);
	    return PyInt_FromLong(l_deleted_count);
	}

	/* -------------------- */
	/* delete number of given lines from END of buffer */
	/* @@@ not implemented, yet */

	/* -------------------- */
	Py_INCREF(Py_None);
	return Py_None;
} /* pyvms_history_delete () */

/* ------------------------------------------------------------------------- */

/* get contents of history buffer as a list of tuples (prompt,command) */

/* l = pyvms.history_get_size() */
PyObject * pyvms_history_get (PyObject *self, PyObject *args)
{
	PyObject			* ar_list;
	PyObject			* ar_tuple;
	PyObject			* ar_prompt;
	PyObject			* ar_command;

	struct pyvms_xr_rlbuffer	* ar_rlbuffer;
	unsigned long			  l_error;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, ""))
	{
	    return NULL;
	}

	/* -------------------- */
	if (pyvms_gar_rlbuffer == NULL)
	{
	    Py_INCREF(Py_None);		/* nothing in the buffer */
	    return Py_None;
	}

	/* -------------------- */
	ar_list = PyList_New(0);	/* create list object */
	if (ar_list == NULL)
	{
	    return NULL;		/* failed */
	}

	ar_rlbuffer = pyvms_gar_rlbuffer;
	while (1)
	{
	    if (ar_rlbuffer == NULL)
	    {
		break;
	    }

	    ar_prompt = PyString_FromString(ar_rlbuffer->at_prompt);
	    if (ar_prompt == NULL)
	    {
		Py_DECREF(ar_list);	/* error, drop list */
		return NULL;
	    }

	    ar_command = PyString_FromString(ar_rlbuffer->at_line);
	    if (ar_command == NULL)
	    {
		Py_DECREF(ar_prompt);	/* error, first element of tuple */
		Py_DECREF(ar_list);	/* error, drop list */
		return NULL;
	    }

	    ar_tuple = PyTuple_New(2);	/* build tuple */
	    if (ar_tuple == NULL)
	    {
		Py_DECREF(ar_prompt);	/* error, first element of tuple */
		Py_DECREF(ar_command);	/* error, second element of tuple */
		Py_DECREF(ar_list);	/* error, drop list */
		return NULL;
	    }

	    PyTuple_SetItem(ar_tuple, 0, ar_prompt);
	    PyTuple_SetItem(ar_tuple, 1, ar_command);

	    l_error = PyList_Append(ar_list, ar_tuple);
	    Py_DECREF(ar_tuple);
	    if (l_error != 0)
	    {
		Py_DECREF(ar_list);	/* error, drop list */
		return NULL;
	    }

	    ar_rlbuffer = ar_rlbuffer->ar_next_rlbuff;
	} /* while (1) */

	return ar_list;
} /* pyvms_history_get () */

/* ------------------------------------------------------------------------- */

/* pyvms.history_show(); */
PyObject * pyvms_history_show (PyObject *self, PyObject *args)
{
	struct pyvms_xr_rlbuffer	* ar_rlbuffer;
	unsigned long			  l_linecount;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, ""))
	{
	    return NULL;
	}

	/* -------------------- */
	if (pyvms_gar_rlbuffer != NULL)
	{
	    l_linecount = 1;
	    ar_rlbuffer = pyvms_gar_rlbuffer;
	    while (1)
	    {
		/* Note: t_line includes '\n' */
/* @@ Py-outroutine */ printf ("%2d %s", l_linecount, ar_rlbuffer->at_line);

		if (ar_rlbuffer->ar_next_rlbuff == NULL)
		{
		    break;
		}

		ar_rlbuffer = ar_rlbuffer->ar_next_rlbuff;
		l_linecount++;
	    }
	}

	/* -------------------- */
	Py_INCREF(Py_None);
	return Py_None;
} /* pyvms_history_show () */

/* ------------------------------------------------------------------------- */

/* n = pyvms.history_size()*/
PyObject * pyvms_history_size (PyObject *self, PyObject *args)
{
	struct pyvms_xr_rlbuffer	* ar_rlbuffer;
	unsigned long			  l_linecount;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, ""))
	{
	    return NULL;
	}

	/* -------------------- */
	if (pyvms_gar_rlbuffer == NULL)
	{
	    l_linecount = 0;		    /* nothing in the buffer */
	}
	else
	{
	    l_linecount = 1;
	    ar_rlbuffer = pyvms_gar_rlbuffer;
	    while (1)
	    {
		if (ar_rlbuffer->ar_next_rlbuff == NULL)
		{
		    break;
		}
		ar_rlbuffer = ar_rlbuffer->ar_next_rlbuff;
		l_linecount++;
	    }
	}

	/* -------------------- */
	return PyInt_FromLong(l_linecount);
} /* pyvms_history_size () */

/* ------------------------------------------------------------------------- */

/* val= pyvms.item_get ('$DVIDEF', 'DVI$_CYLINDERS', None/1); */
PyObject * pyvms_item_get (PyObject *self, PyObject *args)
{
	char			  * at_def_name;
	unsigned long		    l_def_len;
	char			  * at_item_name;
	unsigned long		    l_item_length;
	PyObject		  * ar_option;
	PyObject		  * ar_dict;
	PyObject		  * ar_intobj;
	struct vmsdef_xr_defdirtbl* ar_defdirtbl;
	struct vmsdef_xr_itmtbl	  * ar_itmtbl;

	/* -------------------- */
	ar_option = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "s#s#|O",
	    &at_def_name,  &l_def_len,
	    &at_item_name, &l_item_length,
	    &ar_option))
	{
	    return NULL;
	}

	/* -------------------- */
	/* look up define-name */
	ar_defdirtbl = &VMSDEF_GR_DIRECTORY[0];
	ar_itmtbl    = NULL;
	while (ar_defdirtbl->ar_itmtbl != NULL)
	{
	    if (strcmp(at_def_name,ar_defdirtbl->at_vmsdefnam) == 0)
	    {
		ar_itmtbl = ar_defdirtbl->ar_itmtbl;
		break;
	    }
	    ar_defdirtbl++;
	}

	/* -------------------- */
	if (ar_defdirtbl->ar_itmtbl == NULL)
	{
	    /* item table not found */
	    PyErr_SetString(PyExc_ValueError, "argument 1: unknown DEFine");
	    return NULL;
	}

	/* -------------------- */
	/* look up item code */
	while (ar_itmtbl->w_vmsvermin != 0)
	{
	    /* note: this lookup is independed from version number and    */
	    /*       processor architecture! That's a feature, not a bug! */
		/* @@ unsolved problem: multiple entries of the same item
		   code for several VMS versions */
	    if (strcmp(at_item_name,ar_itmtbl->at_itmnam_py) == 0)
	    {
		/* assume no bitmask is to be returned */
		ar_option = NULL;

		/* check if bitmask bits should be returned */
		/* just test if any argument other than None was given */
		/* all elements of the bitmask are returned for any version */
		if (! (ar_option == Py_None))
		{
		  if (! (ar_itmtbl->l_flags & ITMTBL_M_CVTFNC))
		  {
		    /* ar_itmtbl->ar_bitmask really points to bitmsk */
		    struct vmsdef_x_bitmsk	* ar_bitmsk;
		    PyObject			* ar_list;
		    PyObject			* ar_item;
		    unsigned long		  l_error;

		    /* address of bitmask bit structure */
		    ar_bitmsk = ar_itmtbl->itmtbl_ar_bitmsk;

		    ar_list = PyList_New(0); /* create list object */
		    if (ar_list == NULL)
		    {
		      return NULL;		/* failed */
		    }

		    if (ar_bitmsk != NULL)
		    {
		      /* fill list, if bitmask exists */
		      while (ar_bitmsk->w_vmsvermin != 0)
		      {
			ar_item = PyString_FromString(ar_bitmsk->at_bit_name);
			if (ar_item == NULL)
			{
			    Py_DECREF(ar_list);	/* error, drop list */
			    return NULL;
			}

			l_error = PyList_Append(ar_list, ar_item);
			Py_DECREF(ar_item);
			if (l_error != 0)
			{
			    Py_DECREF(ar_list); /* error, drop list */
			    return NULL;
			}
			ar_bitmsk++;	/* point to next bitmask element */
		      } /* while (ar_bitmsk->w_vmsvermin != 0) */
		      /* @@ optional: status = PyList_Sort(ar_list); */

		      /* return list */
		      ar_option = ar_list;
		    } /* if (ar_bitmsk != NULL) */
		  } /* if (! (ar_itmtbl->l_flags & ITMTBL_M_CVTFNC)) */
		} /* if (! (ar_option == Py_None)) */

		/* return data in a dictionary */
		ar_dict = PyDict_New();
		if (ar_dict == NULL)
		{
		    if (ar_option != NULL)
		    {
			/* drop list, if it has been created */
			Py_XDECREF(ar_option);
		    }
		}

		ar_intobj = PyInt_FromLong((long)ar_itmtbl->w_itmcod);
		PyDict_SetItemString(ar_dict, "item_code", ar_intobj);

		ar_intobj = PyInt_FromLong((long)ar_itmtbl->w_bufsiz);
		PyDict_SetItemString(ar_dict, "buffer_size", ar_intobj);

		ar_intobj = PyInt_FromLong((long)ar_itmtbl->w_vmsvermax);
		PyDict_SetItemString(ar_dict, "vms_version_max", ar_intobj);

		ar_intobj = PyInt_FromLong((long)ar_itmtbl->w_vmsvermin);
		PyDict_SetItemString(ar_dict, "vms_version_min", ar_intobj);

		ar_intobj = PyInt_FromLong( (long)
			(ar_itmtbl->l_flags & ITMTBL_M_ARCH_VAX)   ? 1 : 0);
		PyDict_SetItemString(ar_dict, "applies_to_vax", ar_intobj);

		ar_intobj = PyInt_FromLong( (long)
			(ar_itmtbl->l_flags & ITMTBL_M_ARCH_ALPHA) ? 1 : 0);
		PyDict_SetItemString(ar_dict, "applies_to_alpha", ar_intobj);

		/* @@ ?? w_typ_lib + w_typ_sys -W- unsigned bit! */
		{
		  long l_intobj;

		  if (
		      (ar_itmtbl->itmtbl_ar_bitmsk == NULL) ||
		      (ar_itmtbl->l_flags & ITMTBL_M_CVTFNC)
		     )
		  {
		    l_intobj = 0;
		  }
		  else
		  {
		    l_intobj = 1;
		  }
		  ar_intobj = PyInt_FromLong(l_intobj);
		}
		PyDict_SetItemString(ar_dict, "returns_bitmask", ar_intobj);
		/* @@ returns_constant ? -- cannot currently handled */
		/* VMSDEF structures have no indicator */
		/* @@ new info in struct vmsdef_xr_conmsktbl since 3-JAN-1998*/

		if (ar_option == NULL)
		{
		    ar_option = Py_None;
		    Py_INCREF (ar_option);
		}
		PyDict_SetItemString(ar_dict, "bitmask", ar_option);
		Py_DECREF (ar_option);

		return ar_dict;
	    } /* strcmp(at_item_name,ar_itmtbl->at_itmnam_py) */
	    ar_itmtbl++;		/* next item */
	} /* while (ar_itmtbl->w_vmsvermin != 0) */

	/* -------------------- */
	/* item not found */
	PyErr_SetString(PyExc_ValueError, "argument 2: unknown item");
	return NULL;
} /* pyvms_item_get () */

/* ------------------------------------------------------------------------- */

/* val= pyvms.item_list ('$DVIDEF'); */
PyObject * pyvms_item_list (PyObject *self, PyObject *args)
{
	char			  * at_def_name;
	unsigned long		    l_def_len;
	struct vmsdef_xr_defdirtbl* ar_defdirtbl;
	struct vmsdef_xr_itmtbl	  * ar_itmtbl;
	PyObject		  * ar_list;
	PyObject		  * ar_item;
	unsigned long		    l_error;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "s#", &at_def_name, &l_def_len))
	{
	    return NULL;
	}

	/* -------------------- */
	/* look up define-name */
	ar_defdirtbl = &VMSDEF_GR_DIRECTORY[0];
	ar_itmtbl    = NULL;
	while (ar_defdirtbl->ar_itmtbl != NULL)
	{
	    if (strcmp(at_def_name,ar_defdirtbl->at_vmsdefnam) == 0)
	    {
		ar_itmtbl = ar_defdirtbl->ar_itmtbl;
		break;
	    }
	    ar_defdirtbl++;
	}

	/* -------------------- */
	if (ar_defdirtbl->ar_itmtbl == NULL)
	{
	    /* item table not found */
	    PyErr_SetString(PyExc_ValueError, "argument 1: unknown DEFine");
	    return NULL;
	}

	/* -------------------- */
	/* build a list of all item code names */
	ar_list = PyList_New(0);		/* create list object */
	if (ar_list == NULL)
	{
	    return NULL;			/* failed */
	}

	while (ar_itmtbl->w_vmsvermin != 0)
	{
	    /* note: this lookup is independed from version number and    */
	    /*       processor architecture! That's a feature, not a bug! */

	    ar_item = PyString_FromString(ar_itmtbl->at_itmnam_py);
	    if (ar_item == NULL)
	    {
		Py_DECREF(ar_list);		/* error, drop list */
		return NULL;
	    }
	    l_error = PyList_Append(ar_list, ar_item);
	    Py_DECREF(ar_item);
	    if (l_error != 0)
	    {
		Py_DECREF(ar_list);		/* error, drop list */
		return NULL;
	    }

	    ar_itmtbl++;		/* next item */
	}

	/* -------------------- */
	/* @@ optional: status = PyList_Sort(ar_list); */

	/* -------------------- */
	return ar_list;
} /* pyvms_item_list () */

/* ------------------------------------------------------------------------- */
/* static struct PyMethodDef pyvms_methods[] = {
--	{NULL,		NULL}
-- }; */

/* data is now in file 'PYVMS__METHODS.DAT' */
/* see 'General Manual', configuration guidelines */
extern struct PyMethodDef pyvms__methods[];

/* ---------------------------------------- */

void
initpyvms()
{
	PyObject *m, *d, *l, *s, *v;
	vmsdef_xr_defdirtbl* ar_defdirtbl;
	long		     l_error;

	m = Py_InitModule4("pyvms",
			   pyvms__methods,
			   pyvms__doc__,
			   (PyObject *)NULL,
			   PYTHON_API_VERSION);
        d = PyModule_GetDict(m);

	/* -------------------- */
        if (PyErr_Occurred())
	{
	    (void) Py_FatalError(pyvms_t_cant_init);
	}

	/* -------------------- */
	/* Initialize exception */
	pyvms_gr_error = PyErr_NewException("pyvms.error", NULL, NULL);
	if (pyvms_gr_error == NULL)
	{
	    (void) Py_FatalError(pyvms_t_cant_deferr);
	}
	if (PyDict_SetItemString(d, "error", pyvms_gr_error) != 0)
	{
	    (void) Py_FatalError(pyvms_t_cant_deferr);
	}

	/* -------------------- */
	/* set VMS version number */
	l = PyInt_FromLong((long)vmsdef_gw_vmsver);
	if (l == NULL)
	{
	    (void) Py_FatalError(pyvms_t_cant_defvms);
	}
	if (PyDict_SetItemString(d, "vms_version_number", l) != 0)
	{
	    (void) Py_FatalError(pyvms_t_cant_defvms);
	}

	/* -------------------- */
	/* Initialize vmsdef.definitions list */
	ar_defdirtbl = &VMSDEF_GR_DIRECTORY[0];
	l = PyList_New(0);
	if (l == NULL)
	{
	    return;
	}

	while (ar_defdirtbl->ar_itmtbl != NULL)
	{
	    s = PyString_FromString(ar_defdirtbl->at_vmsdefnam);
	    if (s == NULL)
	    {
		Py_DECREF(l);		/* drop list */
		l = NULL;
		break;
	    }
	    l_error = PyList_Append(l, s);
	    Py_DECREF(s);		/* drop string item */
	    if (l_error != 0)
	    {
		Py_DECREF(l);		/* drop list */
		l = NULL;
		break;
	    }
	    ar_defdirtbl++;
	}
	if (l == NULL)
	{
	    Py_FatalError("can't setup definitions[]");
	}

	s = PyString_FromString("definitions");
	if (s == NULL || PyDict_SetItem (d, s, l) )
	{
	    Py_FatalError("can't setup definitions[]");
	}

	/* -------------------- */
	if (PyErr_Occurred())
	{
	    Py_FatalError("can't initialize module pyvms");
	}

	/* -------------------- */

} /* initpyvms() */

/* ------------------------------------------------------------------------ */

/* EOF: PYVMSMODULE.C */
