/* VMS_LBR_GET_HEADER.C -- 27-SEP-1999 Uwe Zessin
   Python interface to LBR$GET_HEADER
*/

#ifdef __DECC
#pragma module VMS_LBR_GET_HEADER "19991221"
#endif

#include "python.h"
#include "vmsdef.h"

#include <lhidef.h>

/* ------------------------------------------------------------------------- */
extern PyObject *vms_lbr_gr_error;		/* exception vms_lbr.error */
/* ------------------------------------------------------------------------- */

/* ---------------------------------------- */
/* some DEC C versions are missing LBR$ROUTINES.H */
#ifdef __DECC
#ifndef __DECC_VER	/* don't know if this is defined in all versions ... */
#define __DECC_VER 0
#endif /* __DECC_VER */
#if (__DECC_VER < 60090001) /* @@ might need to change for older version */
#ifndef __unknown_params
#define __unknown_params ...
#endif
#pragma __nostandard	  /* non-ANSI-Standard feature */
unsigned long lbr$get_header(__unknown_params);
#pragma __standard
#else
#include <lbr$routines.h>
#endif /* __DECC_VER < 60090001 */
#endif /* __DECC */

/* ------------------------------------------------------------------------- */
/* Generic routine to convert binary data to a Python type. */

extern PyObject * vms__cvt_bin2py
        (char                    * ab_data_addr         /* data address */
        ,unsigned short int        w_data_type          /* data type	*/
        ,unsigned short int        w_bufsiz             /* data length	*/
        );

/* ------------------------------------------------------------------------- */
/* Generic routine to convert a Python type to binary data. */

extern long vms__cvt_py2bin
        (PyObject                * ar_py_data           /* Python data  */
        ,char                    * ab_data_addr         /* data address */
        ,unsigned short int        w_data_type          /* data type    */
        ,unsigned short int        w_bufsiz             /* data length  */
        );

/* ------------------------------------------------------------------------- */

struct vmslbr_x_lhidef {
	char			* at_attrnam;	/* name of attribute */
	short int		  w_offset;	/* offset within NAM */
	unsigned short int	  w_typ;	/* data type	     */
	long			  l_mask;	/* bit address	     */
};
typedef struct vmslbr_x_lhidef vmslbr_x_lhidef;

#define ROAT	0x8000	/* read-only attribute */
#define OBIO	0x4000	/* object I/O */
#define OFSMSK	0x1fff	/* address bits */

static vmslbr_x_lhidef vmslbr_r_lhidef [] = {
	{"L_TYPE",	0,	ITMTYP_K_LONG,1	} ,
	{"L_NINDEX",	4,	ITMTYP_K_LONG,1	} ,
	{"L_MAJORID",	8,	ITMTYP_K_LONG,1	} ,
	{"L_MINORID",	12,	ITMTYP_K_LONG,1	} ,
	{"T_LBRVER",	OBIO+16,ITMTYP_K_ASCIC,1} , /* ASCIC */
	{"L_CREDAT",	48,	ITMTYP_K_QUAD,1	} ,
	{"L_UPDTIM",	56,	ITMTYP_K_QUAD,1	} ,
	{"L_UPDHIS",	64,	ITMTYP_K_LONG,1	} ,
	{"L_FREEVBN",	68,	ITMTYP_K_LONG,1	} ,
	{"L_FREEBLK",	72,	ITMTYP_K_LONG,1	} ,
	{"B_NEXTRFA",	76,	ITMTYP_K_LONG,2	} , /* 2L */
	{"L_NEXTVBN",	84,	ITMTYP_K_LONG,1	} ,
	{"L_FREIDXBLK",	88,	ITMTYP_K_LONG,1	} ,
	{"L_FREEIDX",	92,	ITMTYP_K_LONG,1	} ,
	{"L_HIPREAL",	96,	ITMTYP_K_LONG,1	} ,
	{"L_IDXBLKS",	100,	ITMTYP_K_LONG,1	} ,
	{"L_IDXCNT",	104,	ITMTYP_K_LONG,1	} ,
	{"L_MODCNT",	108,	ITMTYP_K_LONG,1	} ,
	{"L_MHDUSZ",	112,	ITMTYP_K_LONG,1	} ,
	{"L_MAXLUHREC",	116,	ITMTYP_K_LONG,1	} ,
	{"L_NUMLUHREC",	120,	ITMTYP_K_LONG,1	} ,
	{"L_LIBSTATUS",	124,	ITMTYP_K_LONG,1	} ,
	{NULL,  	0,	0		}
};

/* ------------------------------------------------------------------------- */

char vms_lbr_get_header__doc[] =
"retary = vms_lbr.get_header (library_index)\n\
Retrieve Library Header Information.";

/* ---------------------------------------- */

PyObject *
vms_lbr_get_header (PyObject *self, PyObject *args)
{
	unsigned long		  l_library_index;

	unsigned long		  r_lhi[128];

	PyObject		* ar_retary;
	PyObject		* ar_retitm;

	struct vmslbr_x_lhidef	* ar_lhidef;

	char			* ab_data;

	/* size of item types - in PYVMS_GLOBAL.C */
	extern long		  vmsdef_gl_itmtypsiz[];

	unsigned long		  l_status;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "l", &l_library_index))
	{
	    return NULL;
	}

	/* -------------------- */
	(void) memset (&r_lhi, 0, sizeof(r_lhi));

	/* -------------------- */
	l_status = lbr$get_header
		(&l_library_index
		,&r_lhi
		);

	/* -------------------- */
#ifndef LBR$_NORMAL
#define LBR$_NORMAL 2523137
#endif
	if (l_status != LBR$_NORMAL)
	{
	    /* error */
	    return PyVMS_ErrSetVal(vms_lbr_gr_error, 1, l_status);
	}

	/* -------------------- */
	/* create dictionary */
	ar_retary = PyDict_New();
	if (ar_retary == NULL)
	{
	    return NULL;
	}

	/* -------------------- */
	ar_lhidef = &vmslbr_r_lhidef[0];

	while (ar_lhidef->at_attrnam != NULL)
	{
	    ab_data = (char *)&r_lhi;
/*	    printf (":%s\n", ar_lhidef->at_attrnam); */

	    ab_data += (ar_lhidef->w_offset & OFSMSK);

	    /* LHI$T_LBRVER - ASCIC */
	    if ((ar_lhidef->w_offset & OFSMSK) == 16)
	    {
		char		* at_lbrver;
		unsigned char	  b_lbrver_len;

		at_lbrver    = ab_data;
		b_lbrver_len = *at_lbrver;
		at_lbrver++;

		ar_retitm = Py_BuildValue("s#",
				at_lbrver, (unsigned int)b_lbrver_len);
	    }
	    else
	    {
		/* return resultant-value */
		ar_retitm = vms__cvt_bin2py
		    (ab_data			/* data address */
		    ,ar_lhidef->w_typ		/* data type	*/
		    ,vmsdef_gl_itmtypsiz[	/* data length	*/
			ar_lhidef->w_typ
					] * ar_lhidef->l_mask
		    );
	    }

	    if (ar_retitm == NULL)
	    {
		Py_DECREF(ar_retary);
		return NULL;
	    }
	    if (PyDict_SetItemString(ar_retary, ar_lhidef->at_attrnam, ar_retitm)
		< 0)
	    {
		Py_DECREF(ar_retary);
		Py_DECREF(ar_retitm);
		return NULL;
	    }
	    Py_DECREF(ar_retitm);

	    ar_lhidef++;
	} /* while (ar_lhidef->at_attrnam != NULL) */

	/* -------------------- */
	return ar_retary;

} /* vms_lbr_get_header() */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_LBR_GET_HEADER.C */
