/* VMS_LBR_GET_HISTORY.C -- 28-SEP-1999 Uwe Zessin
   Python interface to LBR$GET_HISTORY
*/

#ifdef __DECC
#pragma module VMS_LBR_GET_HISTORY "19991221"
#endif

#include "python.h"
#include "vmsdef.h"

#include <descrip.h>
#include <ssdef.h>

/* ------------------------------------------------------------------------- */
extern PyObject *vms_lbr_gr_error;		/* exception vms_lbr.error */
/* ------------------------------------------------------------------------- */

/* ---------------------------------------- */
/* some DEC C versions are missing LBR$ROUTINES.H */
#ifdef __DECC
#ifndef __DECC_VER	/* don't know if this is defined in all versions ... */
#define __DECC_VER 0
#endif /* __DECC_VER */
#if (__DECC_VER < 60090001) /* @@ might need to change for older version */
#ifndef __unknown_params
#define __unknown_params ...
#endif
#pragma __nostandard	  /* non-ANSI-Standard feature */
unsigned long lbr$get_history(__unknown_params);
#pragma __standard
#else
#include <lbr$routines.h>
#endif /* __DECC_VER < 60090001 */
#endif /* __DECC */

/* ------------------------------------------------------------------------- */
/* Generic routine to convert binary data to a Python type. */

extern PyObject * vms__cvt_bin2py
        (char                    * ab_data_addr         /* data address */
        ,unsigned short int        w_data_type          /* data type	*/
        ,unsigned short int        w_bufsiz             /* data length	*/
        );

/* ------------------------------------------------------------------------- */
/* Generic routine to convert a Python type to binary data. */

extern long vms__cvt_py2bin
        (PyObject                * ar_py_data           /* Python data  */
        ,char                    * ab_data_addr         /* data address */
        ,unsigned short int        w_data_type          /* data type    */
        ,unsigned short int        w_bufsiz             /* data length  */
        );

/* ------------------------------------------------------------------------- */

char vms_lbr_get_history__doc[] =
"status = vms_lbr.get_history (library_index, action_routine)\n\
Retrieve a Library Update History Record.";

/* ---------------------------------------- */

static PyObject * vms_lbr__ar_action_routine; /* Python callback routine */

/* ---------------------------------------- */
static unsigned long vms_lbr__action_routine
    (struct dsc$descriptor_s * ar_history_record)
{
	PyObject * ar_py_history_record;
	PyObject * ar_status;

	ar_py_history_record = Py_BuildValue ("(s#)",
	    ar_history_record->dsc$a_pointer,
	    (unsigned int)ar_history_record->dsc$w_length);
	if (ar_py_history_record == NULL)
	{
	    (void) PyErr_Print();
	    return SS$_ABORT;		/* assume error has been set up */
	}
	ar_status = PyObject_CallObject
		   (vms_lbr__ar_action_routine, ar_py_history_record);
	Py_DECREF(ar_py_history_record);	/* drop this object */
	if (ar_status == NULL)
	{
	    (void) PyErr_Print();
	    return SS$_ABORT;		/* assume error has been set up */
	}
	return SS$_NORMAL;
} /* vms_lbr__action_routine() */

/* ---------------------------------------- */

PyObject *
vms_lbr_get_history (PyObject *self, PyObject *args)
{
	unsigned long		  l_library_index;

	PyObject		* ar_action_routine;
	unsigned long		* al_action_routine;

	unsigned long		  l_status;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "lO",
	    &l_library_index,
	    &ar_action_routine ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: library_index */

	/* -------------------- */
	/* argument 1: action_routine */
	/* argument must be a Python function object */
	if (!PyFunction_Check (ar_action_routine ))
	{
	    PyErr_SetString(PyExc_TypeError,
		"argument 2: action_routine - must be function");
	    return NULL;
	}
	vms_lbr__ar_action_routine = ar_action_routine; /* store object */
	al_action_routine = (unsigned long *)&vms_lbr__action_routine;

	/* -------------------- */
	Py_XINCREF(vms_lbr__ar_action_routine);

	/* -------------------- */
	l_status = lbr$get_history
		(&l_library_index
		,al_action_routine
		);

	/* -------------------- */
	/* release Python function objects */
	Py_XDECREF(vms_lbr__ar_action_routine);

	/* -------------------- */
	return PyInt_FromLong (l_status);
} /* vms_lbr_get_history() */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_LBR_GET_HISTORY.C */
