/* VMS_SMG_CHANGE_VIEWPORT.C -- 05-MAR-2000 Uwe Zessin
   Python interface to SMG$CHANGE_VIEWPORT

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_CHANGE_VIEWPORT

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_CHANGE_VIEWPORT "20000305"
#endif

/* ------------------------------------------------------------------------- */

#include "python.h"
#include "vmsdef.h"

#include <smg$routines.h>	/* SMG$name  */
#include <ssdef.h>		/* SS$_name  */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_change_viewport__doc[] =
"None = vms_smg.change_viewport\
 (display_id, [viewport_row_start], [viewport_column_start],\
 [viewport_number_rows], [viewport_number_columns])\n\
Change the Viewport Associated with a Virtual Display.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_change_viewport (PyObject *self, PyObject *args)
{
	unsigned long		  l_display_id;

	PyObject		* ar_viewport_row_start;
	long			  l_viewport_row_start;
	long			* al_viewport_row_start;

	PyObject		* ar_viewport_column_start;
	long			  l_viewport_column_start;
	long			* al_viewport_column_start;

	PyObject		* ar_viewport_number_rows;
	long			  l_viewport_number_rows;
	long			* al_viewport_number_rows;

	PyObject		* ar_viewport_number_columns;
	long			  l_viewport_number_columns;
	long			* al_viewport_number_columns;

	unsigned long		  l_status;

	/* -------------------- */
	ar_viewport_row_start      = Py_None;
	ar_viewport_column_start   = Py_None;
	ar_viewport_number_rows    = Py_None;
	ar_viewport_number_columns = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|OOOO"
	    ,&l_display_id
	    ,&ar_viewport_row_start
	    ,&ar_viewport_column_start
	    ,&ar_viewport_number_rows
	    ,&ar_viewport_number_columns
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: display_id */

	/* -------------------- */
	/* argument 2: [viewport_row_start] */
	if (ar_viewport_row_start == Py_None)
	{
	    al_viewport_row_start = 0;		/* omitted */
	    l_viewport_row_start  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_viewport_row_start))
	    {
		PyErr_SetString(PyExc_TypeError,
		   "argument 2: viewport_row_start - must be integer or None");
		return NULL;
	    }
	    l_viewport_row_start = PyInt_AsLong(ar_viewport_row_start);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_viewport_row_start = &l_viewport_row_start;
	}

	/* -------------------- */
	/* argument 3: [viewport_column_start] */
	if (ar_viewport_column_start == Py_None)
	{
	    al_viewport_column_start = 0;		/* omitted */
	    l_viewport_column_start  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_viewport_column_start))
	    {
		PyErr_SetString(PyExc_TypeError,
		"argument 3: viewport_column_start - must be integer or None");
		return NULL;
	    }
	    l_viewport_column_start = PyInt_AsLong(ar_viewport_column_start);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_viewport_column_start = &l_viewport_column_start;
	}

	/* -------------------- */
	/* argument 4: [viewport_number_rows] */
	if (ar_viewport_number_rows == Py_None)
	{
	    al_viewport_number_rows = 0;		/* omitted */
	    l_viewport_number_rows  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_viewport_number_rows))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 4: viewport_number_rows - must be integer or None");
		return NULL;
	    }
	    l_viewport_number_rows = PyInt_AsLong(ar_viewport_number_rows);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_viewport_number_rows = &l_viewport_number_rows;
	}

	/* -------------------- */
	/* argument 5: [viewport_number_columns] */
	if (ar_viewport_number_columns == Py_None)
	{
	    al_viewport_number_columns = 0;		/* omitted */
	    l_viewport_number_columns  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_viewport_number_columns))
	    {
		PyErr_SetString(PyExc_TypeError,
	      "argument 5: viewport_number_columns - must be integer or None");
		return NULL;
	    }
	    l_viewport_number_columns = PyInt_AsLong(ar_viewport_number_columns);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_viewport_number_columns = &l_viewport_number_columns;
	}

	/* -------------------- */
	l_status = smg$change_viewport
		(&l_display_id
		,al_viewport_row_start
		,al_viewport_column_start
		,al_viewport_number_rows
		,al_viewport_number_columns
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return (Py_None);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_change_viewport () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_CHANGE_VIEWPORT.C */
