/* VMS_SMG_CREATE_VIRTUAL_DISPLAY.C -- 04-MAR-2000 Uwe Zessin
   Python interface to SMG$CREATE_VIRTUAL_DISPLAY

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_CREATE_VIRTUAL_DISPLAY

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_CREATE_VIRTUAL_DISPLAY "20000304"
#endif

/* ------------------------------------------------------------------------- */

#include "python.h"
#include "vmsdef.h"

#include <smg$routines.h>	/* SMG$name  */
#include <ssdef.h>		/* SS$_name  */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_create_virtual_dpy__doc[] =
"status, display_id\
 = vms_smg.create_virtual_display (number_of_rows, number_of_columns,\
 [display_attributes], [video_attributes], [character_set])\n\
Create a Virtual Display.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_create_virtual_display (PyObject *self, PyObject *args)
{
	unsigned long	l_number_of_rows;
	unsigned long	l_number_of_columns;

	unsigned long	l_display_id;

	PyObject		* ar_display_attributes;
	unsigned long		  l_display_attributes;
	unsigned long		* al_display_attributes;

	PyObject		* ar_video_attributes;
	unsigned long		  l_video_attributes;
	unsigned long		* al_video_attributes;

	PyObject		* ar_character_set;
	unsigned long		  l_character_set;
	unsigned long		* al_character_set;

	unsigned long		  l_status;

	PyObject		* ar_return;

	/* -------------------- */
	ar_display_attributes = Py_None;
	ar_video_attributes   = Py_None;
	ar_character_set      = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "ll|OOO"
	    ,&l_number_of_rows
	    ,&l_number_of_columns
	    ,&ar_display_attributes
	    ,&ar_video_attributes
	    ,&ar_character_set
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: number_of_rows */
	/* -------------------- */
	/* argument 2: number_of_columns */

	/* -------------------- */
	/* argument 3: [display_attributes] */
	if (ar_display_attributes == Py_None)
	{
	    al_display_attributes = 0;		/* omitted */
	    l_display_attributes  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_display_attributes))
	    {
		PyErr_SetString(PyExc_TypeError,
		   "argument 3: display_attributes - must be integer or None");
		return NULL;
	    }
	    l_display_attributes = PyInt_AsLong(ar_display_attributes);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_display_attributes = &l_display_attributes;
	}

	/* -------------------- */
	/* argument 4: [video_attributes] */
	if (ar_video_attributes == Py_None)
	{
	    al_video_attributes = 0;		/* omitted */
	    l_video_attributes  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_video_attributes))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 4: video_attributes - must be integer or None");
		return NULL;
	    }
	    l_video_attributes = PyInt_AsLong(ar_video_attributes);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_video_attributes = &l_video_attributes;
	}

	/* -------------------- */
	/* argument 5: [character_set] */
	if (ar_character_set == Py_None)
	{
	    al_character_set = 0;		/* omitted */
	    l_character_set  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_character_set))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 5: character_set - must be integer or None");
		return NULL;
	    }
	    l_character_set = PyInt_AsLong(ar_character_set);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_character_set = &l_character_set;
	}

	/* -------------------- */
	l_display_id = 0;

	/* -------------------- */
	l_status = smg$create_virtual_display
			(&l_number_of_rows
			,&l_number_of_columns
			,&l_display_id
			,al_display_attributes
			,al_video_attributes
			,al_character_set
			);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    return Py_BuildValue ("ii", l_status, l_display_id);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_create_virtual_display () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_CREATE_VIRTUAL_DISPLAY.C */
