/* VMS_SMG_ERASE_COLUMN.C -- 09-MAR-2000 Uwe Zessin
   Python interface to SMG$ERASE_COLUMN

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_ERASE_COLUMN

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_ERASE_COLUMN "20000309"
#endif

/* ------------------------------------------------------------------------- */

#include "python.h"
#include "vmsdef.h"

#include <smg$routines.h>	/* SMG$name  */
#include <ssdef.h>		/* SS$_name  */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_erase_column__doc[] =
"None = vms_smg.erase_column (display_id, [start_row], [column_number],\
 [end_row])\n\
Erase Column from Display.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_erase_column (PyObject *self, PyObject *args)
{
	unsigned long		  l_display_id;

	PyObject		* ar_start_row;
	unsigned long		  l_start_row;
	unsigned long		* al_start_row;

	PyObject		* ar_column_number;
	unsigned long		  l_column_number;
	unsigned long		* al_column_number;

	PyObject		* ar_end_row;
	unsigned long		  l_end_row;
	unsigned long		* al_end_row;

	unsigned long		  l_status;

	/* -------------------- */
	ar_start_row     = Py_None;
	ar_column_number = Py_None;
	ar_end_row       = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|OOO"
	    ,&l_display_id
	    ,&ar_start_row
	    ,&ar_column_number
	    ,&ar_end_row
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: display_id */

	/* -------------------- */
	/* argument 2: [start_row] */
	if (ar_start_row == Py_None)
	{
	    al_start_row = 0;		/* omitted */
	    l_start_row  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_start_row))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 2: start_row - must be integer or None");
		return NULL;
	    }
	    l_start_row = PyInt_AsLong(ar_start_row);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_start_row = &l_start_row;
	}

	/* -------------------- */
	/* argument 3: [column_number] */
	if (ar_column_number == Py_None)
	{
	    al_column_number = 0;		/* omitted */
	    l_column_number  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_column_number))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: column_number - must be integer or None");
		return NULL;
	    }
	    l_column_number = PyInt_AsLong(ar_column_number);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_column_number = &l_column_number;
	}

	/* -------------------- */
	/* argument 4: [end_row] */
	if (ar_end_row == Py_None)
	{
	    al_end_row = 0;		/* omitted */
	    l_end_row  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_end_row))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 4: end_row - must be integer or None");
		return NULL;
	    }
	    l_end_row = PyInt_AsLong(ar_end_row);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_end_row = &l_end_row;
	}

	/* -------------------- */
	l_status = smg$erase_column
		(&l_display_id
		,al_start_row
		,al_column_number
		,al_end_row
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return (Py_None);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_erase_column () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_ERASE_COLUMN.C */
