/* VMS_SMG_PUT_CHARS_MULTI.C -- 25-MAR-2000 Uwe Zessin
   Python interface to SMG$PUT_CHARS_MULTI

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_PUT_CHARS_MULTI

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_PUT_CHARS_MULTI "20000325"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <smg$routines.h>	/* SMG$name  */
#include <ssdef.h>		/* SS$_name  */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_put_chars_multi__doc[] =
"None = vms_smg.put_chars_multi (display_id, text, [start_row], [start_column],\
 [flags], [rendition_string], [rendition_complement], [character_set])\n\
Put Text with Multiple Renditions to Display.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_put_chars_multi (PyObject *self, PyObject *args)
{
	unsigned long		  l_display_id;

	char			* at_text;
	struct dsc$descriptor_s   r_text;
	unsigned long		  l_text_len;

	PyObject		* ar_start_row;
	long			  l_start_row;
	long			* al_start_row;

	PyObject		* ar_start_column;
	long			  l_start_column;
	long			* al_start_column;

	PyObject		* ar_flags;
	unsigned long		  l_flags;
	unsigned long		* al_flags;

	char			* at_rendition_string;
	struct dsc$descriptor_s   r_rendition_string;
	struct dsc$descriptor_s * ar_rendition_string;
	unsigned long		  l_rendition_string_len;

	PyObject		* ar_rendition_complement;
	unsigned long		  l_rendition_complement;
	unsigned long		* al_rendition_complement;

	PyObject		* ar_character_set;
	unsigned long		  l_character_set;
	unsigned long		* al_character_set;

	unsigned long		  l_status;

	/* -------------------- */
	ar_start_row            = Py_None;
	ar_start_column         = Py_None;
	ar_flags                = Py_None;
	at_rendition_string     = NULL;    l_rendition_string_len = 0;
	ar_rendition_complement = Py_None;
	ar_character_set        = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "is#|OOOz#OO"
	    ,&l_display_id
	    ,&at_text                 ,&l_text_len
	    ,&ar_start_row
	    ,&ar_start_column
	    ,&ar_flags
	    ,&ar_rendition_string
	    ,&ar_rendition_complement
	    ,&ar_character_set
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: display_id */

	/* -------------------- */
	/* argument 2: text */
	if (l_text_len > 65535)
	{
	    PyErr_SetString(PyExc_ValueError,
		"argument 2: text - string size limited to 65535 characters");
	    return NULL;
	}
	r_text.dsc$w_length  = l_text_len;
	r_text.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_text.dsc$b_class   = DSC$K_CLASS_S;
	r_text.dsc$a_pointer = at_text;

	/* -------------------- */
	/* argument 3: [start_row] */
	if (ar_start_row == Py_None)
	{
	    al_start_row = 0;		/* omitted */
	    l_start_row  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_start_row))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: start_row - must be integer or None");
		return NULL;
	    }
	    l_start_row = PyInt_AsLong(ar_start_row);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_start_row = &l_start_row;
	}

	/* -------------------- */
	/* argument 4: [start_column] */
	if (ar_start_column == Py_None)
	{
	    al_start_column = 0;		/* omitted */
	    l_start_column  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_start_column))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 4: column - must be integer or None");
		return NULL;
	    }
	    l_start_column = PyInt_AsLong(ar_start_column);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_start_column = &l_start_column;
	}

	/* -------------------- */
	/* argument 5: [flags] */
	if (ar_flags == Py_None)
	{
	    al_flags = 0;		/* omitted */
	    l_flags  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_flags))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 5: flags - must be integer or None");
		return NULL;
	    }
	    l_flags = PyInt_AsLong(ar_flags);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_flags = &l_flags;
	}

	/* -------------------- */
	/* argument 6: [[rendition_string] */
	if (at_rendition_string == NULL)
	{
	    ar_rendition_string = 0;	/* omitted */
	}
	else
	{
	    if (l_rendition_string_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
     "argument 6: rendition_string - string size limited to 65535 characters");
		return NULL;
	    }
	    r_rendition_string.dsc$w_length  = l_rendition_string_len;
	    r_rendition_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_rendition_string.dsc$b_class   = DSC$K_CLASS_S;
	    r_rendition_string.dsc$a_pointer = at_rendition_string;
	    ar_rendition_string = &r_rendition_string;
	}

	/* -------------------- */
	/* argument 7: [rendition_complement] */
	if (ar_rendition_complement == Py_None)
	{
	    al_rendition_complement = 0;		/* omitted */
	    l_rendition_complement  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_complement))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 7: rendition_complement - must be integer or None");
		return NULL;
	    }
	    l_rendition_complement = PyInt_AsLong(ar_rendition_complement);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_complement = &l_rendition_complement;
	}

	/* -------------------- */
	/* argument 8: [character_set] */
	if (ar_character_set == Py_None)
	{
	    al_character_set = 0;		/* omitted */
	    l_character_set  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_character_set))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 8: character_set - must be integer or None");
		return NULL;
	    }
	    l_character_set = PyInt_AsLong(ar_character_set);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_character_set = &l_character_set;
	}

	/* -------------------- */
	l_status = smg$put_chars_multi
		(&l_display_id
		,&r_text
		,al_start_row
		,al_start_column
		,al_flags
		,ar_rendition_string
		,al_rendition_complement
		,al_character_set
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL) /* @@ SMG$_WILUSERMS */
	{
	    Py_INCREF(Py_None);
	    return (Py_None);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_put_chars_multi () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_PUT_CHARS_MULTI.C */
