/* VMS_SMG_READ_KEYSTROKE.C -- 25-MAR-2000 Uwe Zessin
   Python interface to SMG$READ_KEYSTROKE

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_READ_KEYSTROKE

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_READ_KEYSTROKE "20000325"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_read_keystroke__doc[] =
"status, terminator_code = vms_smg.read_keystroke\
 (keyboard_id, [prompt_string], [timeout], [display_id],\
 [rendition_set], [rendition_complement])\n\
Read a Single Character.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_read_keystroke (PyObject *self, PyObject *args)
{
	unsigned long		  l_keyboard_id;

	char			* at_prompt_string;
	struct dsc$descriptor_s   r_prompt_string;
	struct dsc$descriptor_s * ar_prompt_string;
	unsigned long		  l_prompt_string_len;

	PyObject		* ar_timeout;
	long			  l_timeout;
	long			* al_timeout;

	PyObject		* ar_display_id;
	unsigned long		  l_display_id;
	unsigned long		* al_display_id;

	PyObject		* ar_rendition_set;
	unsigned long		  l_rendition_set;
	unsigned long		* al_rendition_set;

	PyObject		* ar_rendition_complement;
	unsigned long		  l_rendition_complement;
	unsigned long		* al_rendition_complement;

	unsigned short		  w_terminator_code;

	unsigned long		  l_status;

	/* -------------------- */
	at_prompt_string        = NULL;    l_prompt_string_len = 0;
	ar_timeout              = Py_None;
	ar_display_id           = Py_None;
	ar_rendition_set        = Py_None;
	ar_rendition_complement = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|z#OOOO"
	    ,&l_keyboard_id
	    ,&at_prompt_string       ,&l_prompt_string_len
	    ,&ar_timeout
	    ,&ar_display_id
	    ,&ar_rendition_set
	    ,&ar_rendition_complement
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: keyboard_id */

	/* -------------------- */
	/* argument 2: [prompt_string} */
	if (at_prompt_string == NULL)
	{
	    ar_prompt_string = 0;	/* omitted */
	}
	else
	{
	    if (l_prompt_string_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
       "argument 2: prompt_string - string size limited to 65535 characters");
		return NULL;
	    }
	    r_prompt_string.dsc$w_length  = l_prompt_string_len;
	    r_prompt_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_prompt_string.dsc$b_class   = DSC$K_CLASS_S;
	    r_prompt_string.dsc$a_pointer = at_prompt_string;
	    ar_prompt_string = &r_prompt_string;
	}

	/* -------------------- */
	/* argument 3: [timeout] */
	if (ar_timeout == Py_None)
	{
	    al_timeout = 0;		/* omitted */
	    l_timeout  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_timeout))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: timeout - must be integer or None");
		return NULL;
	    }
	    l_timeout = PyInt_AsLong(ar_timeout);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_timeout = &l_timeout;
	}

	/* -------------------- */
	/* argument 4: [display_id] */
	if (ar_display_id == Py_None)
	{
	    al_display_id = 0;		/* omitted */
	    l_display_id  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_display_id))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 4: display_id - must be integer or None");
		return NULL;
	    }
	    l_display_id = PyInt_AsLong(ar_display_id);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_display_id = &l_display_id;
	}

	/* -------------------- */
	/* argument 5: [rendition_set] */
	if (ar_rendition_set == Py_None)
	{
	    al_rendition_set = 0;		/* omitted */
	    l_rendition_set  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_set))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 5: rendition_set - must be integer or None");
		return NULL;
	    }
	    l_rendition_set = PyInt_AsLong(ar_rendition_set);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_set = &l_rendition_set;
	}

	/* -------------------- */
	/* argument 6: [rendition_complement] */
	if (ar_rendition_complement == Py_None)
	{
	    al_rendition_complement = 0;		/* omitted */
	    l_rendition_complement  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_complement))
	    {
		PyErr_SetString(PyExc_TypeError,
		"argument 6: rendition_complement - must be integer or None");
		return NULL;
	    }
	    l_rendition_complement = PyInt_AsLong(ar_rendition_complement);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_complement = &l_rendition_complement;
	}

	/* -------------------- */
	w_terminator_code = 0;

	/* -------------------- */
	l_status = smg$read_keystroke
		(&l_keyboard_id
		,&w_terminator_code
		,ar_prompt_string
		,al_timeout
		,al_display_id
		,al_rendition_set
		,al_rendition_complement
		);

	/* -------------------- */
	return Py_BuildValue ("ii", l_status, (unsigned int)w_terminator_code);

} /* vms_smg_read_keystroke () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_READ_KEYSTROKE.C */
