/* VMS_SMG_SET_CURSOR_REL.C -- 01-APR-2000 Uwe Zessin
   Python interface to SMG$SET_CURSOR_REL

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_SET_CURSOR_REL

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_SET_CURSOR_REL "20000401"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_set_cursor_rel__doc[] =
"None = vms_smg.set_cursor_rel (display_id, [delta_row], [delta_column])\n\
Move Cursor Relative to Current Position.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_set_cursor_rel (PyObject *self, PyObject *args)
{
	unsigned long		  l_display_id;

	PyObject		* ar_delta_row;
	long			  l_delta_row;
	long			* al_delta_row;

	PyObject		* ar_delta_column;
	long			  l_delta_column;
	long			* al_delta_column;

	unsigned long		  l_status;

	/* -------------------- */
	ar_delta_row    = Py_None;
	ar_delta_column = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|OO"
	    ,&l_display_id
	    ,&ar_delta_row
	    ,&ar_delta_column
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: display_id */

	/* -------------------- */
	/* argument 2: [delta_row] */
	if (ar_delta_row == Py_None)
	{
	    al_delta_row = 0;		/* omitted */
	    l_delta_row  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_delta_row))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 2: delta_row - must be integer or None");
		return NULL;
	    }
	    l_delta_row = PyInt_AsLong(ar_delta_row);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_delta_row = &l_delta_row;
	}

	/* -------------------- */
	/* argument 3: [delta_column] */
	if (ar_delta_column == Py_None)
	{
	    al_delta_column = 0;		/* omitted */
	    l_delta_column  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_delta_column))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: delta_column - must be integer or None");
		return NULL;
	    }
	    l_delta_column = PyInt_AsLong(ar_delta_column);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_delta_column = &l_delta_column;
	}

	/* -------------------- */
	l_status = smg$set_cursor_rel
		(&l_display_id
		,al_delta_row
		,al_delta_column
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return (Py_None);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_set_cursor_rel () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_SET_CURSOR_REL.C */
